﻿using Microsoft.CSharp;
using System;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Web.Razor;

namespace Web
{
    class Program
    {
        static void Main(string[] args)
        {
            //配置常量
            string outputAssemblyName = "RazorOutput.dll";
            string className = "CustmerTemplate";
            string baseClassFullName = typeof(TemplateBase).FullName;
            string defaultNamespace = "RazorOutput";
            var tempStr = @"系统时间是: @DateTime.Now";
            //1.创建RazorEngineHost--------------------------------------------------------------
            var razorEngineHost = CreateRazorEngineHost(className, baseClassFullName, defaultNamespace);
            //2.创建RazorTemplateEngine
            var razorTemplateEngine = new RazorTemplateEngine(razorEngineHost);
            //3.解析模板到.Net代码
            var generatorResult = GetTemplateGeneratorResult(razorTemplateEngine, tempStr);
            //4.查看对应的.Net代码
            var generatorResultStr=GetTemplateGeneratorResultStr(generatorResult);
            Console.Write(generatorResultStr);
            //4.编译.Net代码到类库
            var compilerResult = GetCompilerResult(generatorResult, outputAssemblyName);
            if (!compilerResult.Errors.HasErrors)
            {
                //5.执行模板结果
                var templateResult = GetRazorTemplateResult(outputAssemblyName, className, defaultNamespace);
                Console.Write(templateResult);

            }
            Console.ReadLine();
        }

        //创建RazorEngineHost
        static RazorEngineHost CreateRazorEngineHost(string className, string baseClassFullName, string defaultNamespace)
        {
            var language = new CSharpRazorCodeLanguage();
            var razorEngineHost = new RazorEngineHost(language);
            //设置基类名称
            razorEngineHost.DefaultBaseClass = baseClassFullName;
            //设置类名称
            razorEngineHost.DefaultClassName = className;
            //设置命名空间
            razorEngineHost.DefaultNamespace = defaultNamespace;
            //设置引用的程序集
            razorEngineHost.NamespaceImports.Add("System");

            return razorEngineHost;
        }
        //解析模板到.Net代码
        static GeneratorResults GetTemplateGeneratorResult(RazorTemplateEngine razorEngine, string TemplateStr)
        {
            try
            {
                GeneratorResults razorResult = null;
                using (TextReader textReader = new StringReader(TemplateStr))
                {
                    razorResult = razorEngine.GenerateCode(textReader);
                }
                return razorResult;
            }
            catch (Exception)
            {

                throw;
            }

        }
        //查看对应的.Net代码
        static String GetTemplateGeneratorResultStr(GeneratorResults generatorResult) 
        {
            var outputStr=string.Empty;
            var codeProvider = new CSharpCodeProvider();
            using (StringWriter sw = new StringWriter())
            {
                codeProvider.GenerateCodeFromCompileUnit(generatorResult.GeneratedCode, sw, new CodeGeneratorOptions());
                outputStr=sw.GetStringBuilder().ToString();
            }
            return outputStr;
        }
        //编译.Net代码到类库
        static CompilerResults GetCompilerResult(GeneratorResults generatorResult, string outputAssemblyName)
        {
            var codeProvider = new CSharpCodeProvider();
            var baseAssembly = typeof(Program).Assembly.CodeBase.Replace("file:///", "").Replace("/", "\\");
            var compileParams = new CompilerParameters(new string[]{
                baseAssembly
            }, outputAssemblyName);
            // 编译生成的代码到程序集
            return codeProvider.CompileAssemblyFromDom(compileParams, generatorResult.GeneratedCode);
        }
        //得到执行结果
        static string GetRazorTemplateResult(string outputAssemblyName, string className, string nameSpace)
        {
            // 加载程序集
            var assembly = Assembly.LoadFrom(outputAssemblyName);
            var type = assembly.GetType(nameSpace + "." + className);
            var template = Activator.CreateInstance(type) as TemplateBase;
            template.Execute();
            return template.Render();
        }
    }
}
